@@ -1,15 +1,16 @@
 from __future__ import annotations
-from typing import Any, Union
+from typing import Any, Union, Sequence
 import enum
 import copy
 import functools
 from typing import Tuple
-from ..utils import almost_equal
+from ..utils import almost_equal, areinstances, check_foreach
 from .Complex import Complex
 from .Vector import Vector
 from .Field import Field, RealField
-from ..utils import areinstances, check_foreach, isoneof
 from .Span import Span
+from danielutils import validate, isoneof, NotImplemented
+from ..BaseClasses import Matrix____
 
 
 class MatrixOperationType(enum.Enum):
@@ -24,9 +25,298 @@ class MatrixOperationType(enum.Enum):
 MOT = MatrixOperationType
 
 
-class Matrix:
+class Matrix__(Matrix____):
+    @validate(None, Sequence, Field)
+    def __init__(self, mat: list[list[Any]], field: Field = RealField()) -> None:
+        """Create a Matrix
+
+        Args:
+            mat (list[list[Any]]): a list of lists of elements (2D array)
+            field (Field, optional): the field that the elements of the matrix is over. Defaults to RealField().
+
+        Raises:
+            TypeError: will rise if mat is not a list of lists
+        """
+        if not isinstance(mat, list) or not all([isinstance(v, list) for v in mat]):
+            raise TypeError("Matrix must be a 2d array")
+        self._matrix = mat
+        self._rows = len(mat)
+        self._cols = len(mat[0])
+        self.field = field
+
+    @validate(None, int)
+    def __getitem__(self, index: int) -> list[Any]:
+        """wil return the row at the given index
+
+        Args:
+            index (int): the index of desire row
+
+        Raises:
+            TypeError: will rise if index is not an int
+            ValueError: will rise if index is out of range
+
+        Returns:
+            list[Any]: the row at the given index
+        """
+        if not (0 <= index < self._rows):
+            raise ValueError("Index out of range")
+        return self._matrix[index]
+
+    @validate(None, int, list)
+    def __setitem__(self, index: int, value: list[Any]) -> None:
+        if not 0 <= index < self._rows:
+            raise ValueError("Index out of range")
+        # FIXME validate all items in list are of same type of this matrix
+        self._matrix[index] = value
+
+    @validate(None, int)
+    def __str__(self, turnc: int = 2) -> str:
+        """will return a string representation of the matrix
+
+        Args:
+            turnc (int, optional): how many digist to turnicate. Defaults to 2.
+
+        Returns:
+            str: the srtring representation
+        """
+        def round_if_possible(v):
+            if hasattr(v, '__round__'):
+                if almost_equal(round(v), v):
+                    v = round(v)
+            return v
+
+        def turnacate(v):
+            v = str(v)
+            return v[:v.index(".")+1+turnc] if "." in v else v
+
+        def find_spaceing():
+            res = 0
+            for row in self:
+                for v in row:
+                    v = round_if_possible(v)
+                    v = turnacate(v)
+                    res = max(res, len(str(v)))
+            return res
+        spacing = find_spaceing()+2
+        n = len(self[0])
+        vl = "|"
+        hl = (1+spacing)*n*"-" + "-" + "\n"
+        result = hl
+        for i, row in enumerate(self._matrix):
+            result += vl
+            for v in row:
+                v = round_if_possible(v)
+                v = turnacate(v)
+                result += str(v).center(spacing)+vl
+            result += "\n"+hl
+        return result
+
+    @validate(None, Matrix____)
+    def __add__(self, other: Matrix__) -> Matrix__:
+        """will add two matrices together and return the result
+
+        Args:
+            other (Matrix): the matrix to add to current one
+
+        Raises:
+            TypeError: will rise if other is not a matrix
+            ValueError: will rise if the matrices are not of the same size
+
+        Returns:
+            Matrix: the result matrix
+        """
+        if not isinstance(other, Matrix):
+            raise TypeError("Matrix can only be added to another Matrix")
+        if self._rows != other._rows or self._cols != other._cols:
+            raise ValueError("Matrices must have the same dimensions")
+        return Matrix([[self._matrix[i][j] + other._matrix[i][j] for j in range(self._cols)]
+                       for i in range(self._rows)])
+
+    def __neg__(self) -> Matrix__:
+        """will return the negative of the matrix
+
+        Returns:
+            Matrix: the negative matrix
+        """
+        return Matrix([[-self._matrix[i][j] for j in range(self._cols)]
+                       for i in range(self._rows)])
+
+    @validate(None, Matrix____)
+    def __sub__(self, other: Matrix__) -> Matrix__:
+        """will subtract two matrices and return the result
+
+        Args:
+            other (Matrix): the matrix to subtract from current one
+
+        Raises:
+            TypeError: will rise if other is not a matrix
+            ValueError: will rise if the matrices are not of the same size
+
+        Returns:
+            Matrix: the result matrix
+        """
+        if not isinstance(other, Matrix):
+            raise TypeError(
+                "Matrix can only be subtracted from another Matrix")
+        if self._rows != other._rows or self._cols != other._cols:
+            raise ValueError("Matrices must have the same dimensions")
+        return Matrix([[self._matrix[i][j] - other._matrix[i][j] for j in range(self._cols)]
+                       for i in range(self._rows)])
+
+    def __mul__(self, other: Any) -> Union[Matrix__, Vector]:
+        """will multiply the matrix with the given value and return the result
+
+        Args:
+            other (Union[int, float, Complex, Vector, Matrix, Calculable]): the value to multiply with
+
+        Raises:
+            ValueError: will rise if the value is not one of the above types
+
+        Returns:
+            Any: [int, float, Complex, Vector, Matrix, Calculable]
+        """
+        from ..la2 import Calculable
+        if not isoneof(other, [int, float, complex, Vector, Matrix, Calculable]):
+            raise ValueError(
+                "Can only multiply with a int, float, complex, Vector, Matrix or Calculable")
+        if isoneof(other, [int, float, Complex, Calculable]):
+            res: list[list[Any]] = []
+            for i in range(len(self)):
+                res.append([])
+                for j in range(len(self[0])):
+                    res[i].append(self[i][j] * other)
+            return Matrix(res)
+        if isinstance(other, Vector):
+            if self._cols != len(other):
+                raise ValueError(
+                    "Matrix and Vector must have the same number of rows")
+            return Vector([sum([self._matrix[i][j] * other[j] for j in range(self._cols)])
+                           for i in range(self._rows)])
+        if isinstance(other, Matrix):
+            if self._cols != other._rows:
+                raise ValueError(
+                    "Matrix and Matrix must have matching sizes: self.cols == other.rows")
+            return Matrix([[sum([self._matrix[i][j] * other._matrix[j][k] for j in range(self._cols)])
+                            for k in range(other._cols)] for i in range(self._rows)])
+
+    def __rmul__(self, other: Union[int, float, Complex, Vector, Matrix__]) -> Union[float, Complex, Vector, Matrix__]:
+        """will multiply the matrix with the given value and return the result
+
+        Args:
+            other (Union[int, float, Complex, Vector, Matrix, Calculable]): the value to multiply with
+
+        Raises:
+            ValueError: will rise if the value is not one of the above types
+
+        Returns:
+            Any: [int, float, Complex, Vector, Matrix, Calculable]
+        """
+        if isoneof(other, [int, float, Complex]):
+            return self.__mul__(other)
+        if isinstance(other, Vector):
+            raise TypeError(
+                "Matrix can only be multiplied by a vector from the right")
+        if isinstance(other, Matrix):
+            if self._cols != other._rows:
+                raise ValueError(
+                    "Matrix and Matrix must have the same number of columns")
+            return Matrix([[sum([self._matrix[i][j] * other._matrix[j][k] for j in range(self._cols)])
+                            for k in range(other._cols)] for i in range(self._rows)])
+        raise TypeError(
+            f"cant perform {type(other)}*Matrix.\ncan only be multiplied by a:\n\tint\n\tfloat\n\tComplex\n\tVector\n\tMatrix]")
+
+    @validate(None, Matrix____, bool)
+    def __eq__(self, other: Matrix__, use_almost_equale: bool = True) -> bool:
+        """will compare two matrices and return True if they are equal
+
+        Args:
+            other (Matrix): the other matrix to compare to
+            use_almost_equale (bool, optional): Specifies wheter to use almost_equal when comparing Matricies. Defaults to True.
+
+        Raises:
+            TypeError: will rise it the other object is not a Matrix
+            TypeError: wiil rise if use_almost_equale is not a bool
+
+        Returns:
+            bool: True if the objects are equal, False otherwise
+        """
+        if not isinstance(other, Matrix):
+            raise TypeError(f"cant complare 'Matrix' with '{type(other)}'")
+        if not isinstance(use_almost_equale, bool):
+            raise TypeError("use_almost_equale must be a boolean")
+        if self._rows != other._rows or self._cols != other._cols:
+            return False
+        if use_almost_equale:
+            for i in range(len(self)):
+                for j in range(len(self[i])):
+                    if not almost_equal(self[i][j], other[i][j]):
+                        return False
+        if any([self._matrix[i][j] != other._matrix[i][j] for i in range(self._rows) for j in range(self._cols)]):
+            return False
+        return True
+
+    @validate(None, Matrix____, bool)
+    def __ne__(self, other: Matrix__, use_almost_equale: bool = True) -> bool:
+        """will compare two matrices and return True if they are not equal
+
+        Args:
+            other (Matrix): the other matrix to compare to
+            use_almost_equale (bool, optional): Specifies wheter to use almost_equal when comparing Matricies. Defaults to True.
+
+        Raises:
+            TypeError: will rise if __eq__ would raise an error
+
+        Returns:
+            bool: True if the objects are not equal, False otherwise
+        """
+        try:
+            return not self.__eq__(other, use_almost_equale)
+        except TypeError as e:
+            raise e
+
+    def __len__(self) -> int:
+        """will return the number of rows in the matrix
+
+        Returns:
+            int: the number of rows in the matrix
+        """
+        return self._rows
+
+    @validate(None, int)
+    def __pow__(self, value: int) -> Matrix__:
+        """will raise the matrix to the given power and return the result in a new matrix
+
+        Args:
+            value (int): the power to raise the matrix to
+
+        Raises:
+            TypeError: will rise if value is not an int
+            ValueError: will rise if value is less than 0
+
+        Returns:
+            Matrix: the result of the power
+        """
+        if isinstance(value, float):
+            if int(value) != value:
+                raise ValueError("value must be an integer")
+            value = int(value)
+        if not isinstance(value, int):
+            raise TypeError("value must be an int")
+        if not (0 <= value):
+            raise ValueError("value must be atleast than 0")
+        res = self
+        for _ in range(value-1):
+            res *= self
+        return res
+
+    def __iter__(self) -> list:
+        return iter(self._matrix)
+
+
+class Matrix(Matrix__):
 
     @staticmethod
+    @validate(Vector)
     def fromVector(vec: Vector) -> Matrix:
         """Create a Matrix from a single Vector, the matrix will have 1 column and the same number of rows as the vector
 
@@ -44,7 +334,8 @@ class Matrix:
         return Matrix([[v] for v in vec], vec.field)
 
     @staticmethod
-    def from_vectors(vecs: list[Vector]) -> Matrix:
+    @validate([Sequence, lambda seq: areinstances(seq, Vector), "all elements must be instances of class 'Vector'"])
+    def from_vectors(vecs: Sequence[Vector]) -> Matrix:
         """Create a Matrix from a list of Vectors, the matrix will have the same number of columns as the number of vectors and the same number of rows as the number of elements in a vector
 
         Args:
@@ -57,8 +348,6 @@ class Matrix:
         Returns:
             Matrix: the result
         """
-        if not areinstances(vecs, Vector):
-            raise TypeError("all elements must be instances of class 'Vector'")
         if not check_foreach(vecs, lambda v: v.field == vecs[0].field):
             raise ValueError("vectors are not over the same field")
         mat: list[list[Any]] = []
@@ -68,20 +357,8 @@ class Matrix:
                 mat[i].append(vecs[j][i])
         return Matrix(mat, field=vecs[0].field)
 
-    # @staticmethod
-    # def fromString(matrix_string: str) -> Matrix:
-    #     """Create a Matrix from a string representation of a matrix
-
-    #     Args:
-    #         matrix_string (str): the sr
-
-    #     Returns:
-    #         Matrix: _description_
-    #     """
-    #     return Matrix([[int(num) for num in row.split()]
-    #                    for row in matrix_string.split("\n")])
-
     @staticmethod
+    @validate(int, int, Field, float, float)
     def random(rows: int, cols: int, f: Field = RealField(), min: float = -10, max: float = 10, ) -> Matrix:
         """Create a random Matrux acording to params
 
@@ -95,11 +372,11 @@ class Matrix:
         Returns:
             Matrix: the result
         """
-
         return Matrix([[f.random(min, max) for _ in range(cols)]for __ in range(rows)], field=f)
 
     @staticmethod
-    def from_jordan_blocks(blocks: list[Matrix]) -> Matrix:
+    @validate([Sequence, lambda seq: areinstances(seq, Matrix), None])
+    def from_jordan_blocks(blocks: Sequence[Matrix]) -> Matrix:
         """Create a Matrix from a list of Jordan blocks
 
         Args:
@@ -108,28 +385,42 @@ class Matrix:
         Returns:
             Matrix: the result
         """
-        total_size = 0
-        hm: dict[Any, list[Matrix]] = dict()
-        for block in blocks:
-            if block[0][0] not in hm.keys():
-                hm[block[0][0]] = []
-            hm[block[0][0]].append(block)
-            total_size += len(block)
-        # sort blocks in order
-        for key in hm:
-            hm[key].sort(key=lambda x: len(x))
+        # group blocks by eigenvectors
+        eigenvalues = set([m[0][0] for m in blocks])
+        matricies: list[list[Matrix]] = []
+        for eigenvalue in eigenvalues:
+            matricies.append([])
+            for m in blocks:
+                if m[0][0] == eigenvalue:
+                    matricies[-1].append(m)
+
+        def inside_sorter(b1, b2):
+            size1 = len(b1)
+            size2 = len(b2)
+            return size2-size1
+        # sort each sub list for big to small
+        for i in range(len(eigenvalues)):
+            matricies[i].sort(key=functools.cmp_to_key(inside_sorter))
+        # rebuild blocks array
+        blocks = []
+        for mat_list in matricies:
+            blocks.extend(mat_list)
+        # calculate final size
+        total_size = sum([len(m) for m in blocks])
+        # initialize result
         res = Matrix([[0 for __ in range(total_size)] for _ in range(total_size)],
                      field=blocks[0].field)
+        # set values inside result
         offset = 0
-        for key in hm:
-            for block in hm[key]:
-                for i, row in enumerate(block):
-                    for j, v in enumerate(row):
-                        res[i + offset][j + offset] = v
-                offset += len(block)
+        for block in blocks:
+            for i in range(len(block)):
+                for j in range(len(block)):
+                    res[i+offset][j+offset] = block[i][j]
+            offset += len(block)
         return res
 
     @staticmethod
+    @validate(int, None)
     def create_jordan_blcok(size: int, eigenvalue: Any) -> Matrix:
         """Create a Jordan block from a size and an eigenvalue
 
@@ -146,6 +437,7 @@ class Matrix:
         return Matrix.identity(size)*eigenvalue + m
 
     @staticmethod
+    @validate([int, lambda x: x > 0, "must be positive integer"])
     def identity(size: int) -> Matrix:
         """Create identity matrix if given size
 
@@ -160,23 +452,6 @@ class Matrix:
             arr[i][i] = 1
         return Matrix(arr)
 
-    def __init__(self, mat: list[list[Any]], field: Field = RealField()) -> None:
-        """Create a Matrix
-
-        Args:
-            mat (list[list[Any]]): a list of lists of elements (2D array)
-            field (Field, optional): the field that the elements of the matrix is over. Defaults to RealField().
-
-        Raises:
-            TypeError: will rise if mat is not a list of lists
-        """
-        if not isinstance(mat, list) or not all([isinstance(v, list) for v in mat]):
-            raise TypeError("Matrix must be a 2d array")
-        self.__matrix = mat
-        self.__rows = len(mat)
-        self.__cols = len(mat[0])
-        self.field = field
-
     @property
     def kernel(self) -> Union[Vector, Span]:
         """Get the kernel of the matrix
@@ -199,6 +474,26 @@ class Matrix:
         from .VectorSpace import VectorSpace
         return VectorSpace(self.field).standard_basis() - self.kernel
 
+    @property
+    @NotImplemented
+    def adjugate(self) -> Matrix:
+        pass
+        # TODO implement adjugate
+
+    @property
+    def row_space(self) -> list[Vector]:
+        return [Vector(r) for r in self]
+
+    @property
+    def column_space(self) -> list[Vector]:
+        res: list[Vector] = []
+        for j in range(len(self[0])):
+            arr = []
+            for i in range(len(self)):
+                arr.append(self[i][j])
+            res.append(Vector(arr))
+        return res
+
     @property
     def rank(self) -> int:
         """Get the rank of the matrix
@@ -230,13 +525,13 @@ class Matrix:
         Returns:
             Any: the result
         """
-        if self.__rows != self.__cols:
+        if self._rows != self._cols:
             raise ValueError("Matrix must be square")
-        if self.__rows == 1:
-            return self.__matrix[0][0]
-        if self.__rows == 2:
-            return self.__matrix[0][0] * self.__matrix[1][1] - self.__matrix[0][1] * self.__matrix[1][0]
-        return sum([self.__matrix[i][0] * ((-1)**i) * self.minor(i, 0) for i in range(self.__rows)])
+        if self._rows == 1:
+            return self._matrix[0][0]
+        if self._rows == 2:
+            return self._matrix[0][0] * self._matrix[1][1] - self._matrix[0][1] * self._matrix[1][0]
+        return sum([self._matrix[i][0] * ((-1)**i) * self.minor(i, 0) for i in range(self._rows)])
 
     @property
     def is_invertiable(self) -> bool:
@@ -247,6 +542,14 @@ class Matrix:
         """
         return self.determinant != 0
 
+    @property
+    def inverse(self) -> Matrix:
+
+        if not self.is_invertiable:
+            raise ValueError("Matrix must be invertible")
+        _, res = self.gaussian_elimination_with(Matrix.identity(len(self)))
+        return res
+
     @property
     def is_square(self) -> bool:
         """returns wheter the matrix is square
@@ -254,7 +557,7 @@ class Matrix:
         Returns:
             bool: True if the matrix is square, False otherwise
         """
-        return self.__rows == self.__cols
+        return self._rows == self._cols
 
     @property
     def is_symmetrical(self) -> bool:
@@ -302,6 +605,10 @@ class Matrix:
                 return False
         return True
 
+    @property
+    def is_projection(self) -> bool:
+        return self**2 == self
+
     @property
     def is_nilpotent(self) -> bool:
         # TODO implement nilpotency check
@@ -328,19 +635,26 @@ class Matrix:
         """
         if not self.is_square:
             raise ValueError("Matrix must be square")
+        Block = Matrix.from_jordan_blocks
+        J = Matrix.create_jordan_blcok
         blocks = []
         eigenvalues = self.eigenvalues
+        V0 = Vector([0 for _ in range(len(self))])
+        In = Matrix.identity(len(self))
         for eigenvalue in set(eigenvalues):
-            geom = self.geometric_multiplicity(eigenvalue)
-            alge = eigenvalues.count(eigenvalue)
-            if geom == 1:
-                blocks.append(self.create_jordan_blcok(alge, eigenvalue))
-            elif geom == alge:
-                blocks.append(eigenvalue*Matrix.identity(alge))
+            alg = eigenvalues.count(eigenvalue)
+            counts = []
+            M_lamda: Matrix = self-eigenvalue*In
+            for k in range(1, alg):
+                counts.append((M_lamda**k).kernel.dim)
+            if len(counts) == 1:
+                if counts[0] == 1:
+                    blocks.append(J(alg, eigenvalue))
             else:
-                # alge>geom => chain_basis
+                diagram = [[0 for _ in range(counts[0])]
+                           for __ in range(counts[-1])]
                 pass
-        return Matrix.from_jordan_blocks(blocks)
+        return Block(blocks)
 
     @property
     def chain_basis(self) -> list[Vector]:
@@ -371,278 +685,41 @@ class Matrix:
         """
         from ..la2 import PolynomialSimple
         res = PolynomialSimple([1], [0])
-        for eigenvalue in set(self.eigenvalues):
-            res *= PolynomialSimple([1, -eigenvalue], [1, 0]
-                                    )**self.geometric_multiplicity(eigenvalue)
+        eigenvalues = self.eigenvalues
+        for eigenvalue in set(eigenvalues):
+            alg = eigenvalues.count(eigenvalue)
+            tmp_matrix: Matrix = (
+                self-eigenvalue*Matrix.identity(len(self)))
+            power = 0
+            ker = tmp_matrix.kernel
+            if isinstance(ker, Span):
+                if ker.dim == 1:
+                    power = alg
+                elif ker.dim == alg:
+                    power = 1
+                else:
+                    # M0 = Matrix.identity(len(self))*0
+                    # print(M0)
+                    # original = tmp_matrix
+                    # power += 1
+                    # print(tmp_matrix)
+                    # while tmp_matrix != M0:
+                    #     tmp_matrix *= original
+                    #     print(tmp_matrix)
+                    #     power += 1
+                    _0 = Vector([0 for _ in range(len(ker[0]))])
+                    for v in ker:
+                        curr_power = 1
+                        u = tmp_matrix*v
+                        while u != _0:
+                            u = tmp_matrix*v
+                            curr_power += 1
+                        power = max(power, curr_power)
+
+            res *= PolynomialSimple([1, -eigenvalue], [1, 0])**power
         return res
 
-    def __getitem__(self, index: int) -> list[Any]:
-        """wil return the row at the given index
-
-        Args:
-            index (int): the index of desire row
-
-        Raises:
-            TypeError: will rise if index is not an int
-            ValueError: will rise if index is out of range
-
-        Returns:
-            list[Any]: the row at the given index
-        """
-        if not isinstance(index, int):
-            raise TypeError("Index must be an integer")
-        if not (0 <= index <= self.__rows):
-            raise ValueError("Index out of range")
-        return self.__matrix[index]
-
-    def __setitem__(self, index: int, value: list[Any]) -> None:
-        if not isinstance(index, int):
-            raise TypeError("Index must be an integer")
-        if not 0 <= index < self.__rows:
-            raise ValueError("Index out of range")
-        if not isinstance(value, list):
-            raise TypeError("Value must be a list")
-        # FIXME validate all items in list are of same type of this matrix
-        self.__matrix[index] = value
-
-    def __str__(self, turnc: int = 2) -> str:
-        """will return a string representation of the matrix
-
-        Args:
-            turnc (int, optional): how many digist to turnicate. Defaults to 2.
-
-        Returns:
-            str: the srtring representation
-        """
-        def round_if_possible(v):
-            if hasattr(v, '__round__'):
-                if almost_equal(round(v), v):
-                    v = round(v)
-            return v
-
-        def turnacate(v):
-            v = str(v)
-            return v[:v.index(".")+1+turnc] if "." in v else v
-
-        def find_spaceing():
-            res = 0
-            for row in self:
-                for v in row:
-                    v = round_if_possible(v)
-                    v = turnacate(v)
-                    res = max(res, len(str(v)))
-            return res
-        spacing = find_spaceing()+2
-        n = len(self[0])
-        vl = "|"
-        hl = (1+spacing)*n*"-" + "-" + "\n"
-        result = hl
-        for i, row in enumerate(self.__matrix):
-            result += vl
-            for v in row:
-                v = round_if_possible(v)
-                v = turnacate(v)
-                result += str(v).center(spacing)+vl
-            result += "\n"+hl
-        return result
-
-    def __add__(self, other: Matrix) -> Matrix:
-        """will add two matrices together and return the result
-
-        Args:
-            other (Matrix): the matrix to add to current one
-
-        Raises:
-            TypeError: will rise if other is not a matrix
-            ValueError: will rise if the matrices are not of the same size
-
-        Returns:
-            Matrix: the result matrix
-        """
-        if not isinstance(other, Matrix):
-            raise TypeError("Matrix can only be added to another Matrix")
-        if self.__rows != other.__rows or self.__cols != other.__cols:
-            raise ValueError("Matrices must have the same dimensions")
-        return Matrix([[self.__matrix[i][j] + other.__matrix[i][j] for j in range(self.__cols)]
-                       for i in range(self.__rows)])
-
-    def __neg__(self) -> Matrix:
-        """will return the negative of the matrix
-
-        Returns:
-            Matrix: the negative matrix
-        """
-        return Matrix([[-self.__matrix[i][j] for j in range(self.__cols)]
-                       for i in range(self.__rows)])
-
-    def __sub__(self, other: Matrix) -> Matrix:
-        """will subtract two matrices and return the result
-
-        Args:
-            other (Matrix): the matrix to subtract from current one
-
-        Raises:
-            TypeError: will rise if other is not a matrix
-            ValueError: will rise if the matrices are not of the same size
-
-        Returns:
-            Matrix: the result matrix
-        """
-        if not isinstance(other, Matrix):
-            raise TypeError(
-                "Matrix can only be subtracted from another Matrix")
-        if self.__rows != other.__rows or self.__cols != other.__cols:
-            raise ValueError("Matrices must have the same dimensions")
-        return Matrix([[self.__matrix[i][j] - other.__matrix[i][j] for j in range(self.__cols)]
-                       for i in range(self.__rows)])
-
-    def __mul__(self, other: Any) -> Any:
-        """will multiply the matrix with the given value and return the result
-
-        Args:
-            other (Union[int, float, Complex, Vector, Matrix, Calculable]): the value to multiply with
-
-        Raises:
-            ValueError: will rise if the value is not one of the above types
-
-        Returns:
-            Any: [int, float, Complex, Vector, Matrix, Calculable]
-        """
-        from ..la2 import Calculable
-        if not isoneof(other, [int, float, complex, Vector, Matrix, Calculable]):
-            raise ValueError(
-                "Can only multiply with a int, float, complex, Vector, Matrix or Calculable")
-        if isoneof(other, [int, float, Complex, Calculable]):
-            res: list[list[Any]] = []
-            for i in range(len(self)):
-                res.append([])
-                for j in range(len(self[0])):
-                    res[i].append(self[i][j] * other)
-            return Matrix(res)
-        if isinstance(other, Vector):
-            if self.__cols != len(other):
-                raise ValueError(
-                    "Matrix and Vector must have the same number of rows")
-            return Vector([sum([self.__matrix[i][j] * other[j] for j in range(self.__cols)])
-                           for i in range(self.__rows)])
-        if isinstance(other, Matrix):
-            if self.__cols != other.__rows:
-                raise ValueError(
-                    "Matrix and Matrix must have matching sizes: self.cols == other.rows")
-            return Matrix([[sum([self.__matrix[i][j] * other.__matrix[j][k] for j in range(self.__cols)])
-                            for k in range(other.__cols)] for i in range(self.__rows)])
-
-    def __rmul__(self, other: Union[int, float, Complex, Vector, Matrix]) -> Union[float, Complex, Vector, Matrix]:
-        """will multiply the matrix with the given value and return the result
-
-        Args:
-            other (Union[int, float, Complex, Vector, Matrix, Calculable]): the value to multiply with
-
-        Raises:
-            ValueError: will rise if the value is not one of the above types
-
-        Returns:
-            Any: [int, float, Complex, Vector, Matrix, Calculable]
-        """
-        if isoneof(other, [int, float, Complex]):
-            return self.__mul__(other)
-        if isinstance(other, Vector):
-            raise TypeError(
-                "Matrix can only be multiplied by a vector from the right")
-        if isinstance(other, Matrix):
-            if self.__cols != other.__rows:
-                raise ValueError(
-                    "Matrix and Matrix must have the same number of columns")
-            return Matrix([[sum([self.__matrix[i][j] * other.__matrix[j][k] for j in range(self.__cols)])
-                            for k in range(other.__cols)] for i in range(self.__rows)])
-        raise TypeError(
-            f"cant perform {type(other)}*Matrix.\ncan only be multiplied by a:\n\tint\n\tfloat\n\tComplex\n\tVector\n\tMatrix]")
-
-    def __eq__(self, other: Matrix, use_almost_equale: bool = True) -> bool:
-        """will compare two matrices and return True if they are equal
-
-        Args:
-            other (Matrix): the other matrix to compare to
-            use_almost_equale (bool, optional): Specifies wheter to use almost_equal when comparing Matricies. Defaults to True.
-
-        Raises:
-            TypeError: will rise it the other object is not a Matrix
-            TypeError: wiil rise if use_almost_equale is not a bool
-
-        Returns:
-            bool: True if the objects are equal, False otherwise
-        """
-        if not isinstance(other, Matrix):
-            raise TypeError(f"cant complare 'Matrix' with '{type(other)}'")
-        if not isinstance(use_almost_equale, bool):
-            raise TypeError("use_almost_equale must be a boolean")
-        if self.__rows != other.__rows or self.__cols != other.__cols:
-            return False
-        if use_almost_equale:
-            return all([all([almost_equal(self[i][j], other[i][j])] for j in range(len(self[0]))) for i in range(len(self))])
-        if any([self.__matrix[i][j] != other.__matrix[i][j] for i in range(self.__rows) for j in range(self.__cols)]):
-            return False
-        return True
-
-    def __ne__(self, other: Matrix, use_almost_equale: bool = True) -> bool:
-        """will compare two matrices and return True if they are not equal
-
-        Args:
-            other (Matrix): the other matrix to compare to
-            use_almost_equale (bool, optional): Specifies wheter to use almost_equal when comparing Matricies. Defaults to True.
-
-        Raises:
-            TypeError: will rise if __eq__ would raise an error
-
-        Returns:
-            bool: True if the objects are not equal, False otherwise
-        """
-        try:
-            return not self.__eq__(other, use_almost_equale)
-        except TypeError as e:
-            raise e
-
-    def __len__(self) -> int:
-        """will return the number of rows in the matrix
-
-        Returns:
-            int: the number of rows in the matrix
-        """
-        return self.__rows
-
-    def __pow__(self, value: int) -> Matrix:
-        """will raise the matrix to the given power and return the result in a new matrix
-
-        Args:
-            value (int): the power to raise the matrix to
-
-        Raises:
-            TypeError: will rise if value is not an int
-            ValueError: will rise if value is less than 0
-
-        Returns:
-            Matrix: the result of the power
-        """
-        if isinstance(value, float):
-            if int(value) != value:
-                raise ValueError("value must be an integer")
-            value = int(value)
-        if not isinstance(value, int):
-            raise TypeError("value must be an int")
-        if not (0 <= value):
-            raise ValueError("value must be atleast than 0")
-        res = self
-        for _ in range(value-1):
-            res *= self
-        return res
-
-    def inverse(self) -> Matrix:
-        pass
-        # if not self.is_invertiable:
-        #     raise ValueError("Matrix must be invertible")
-        # return Matrix([[self.minor(i, j) / self.determinant for j in range(self.__cols)]
-        #                for i in range(self.__rows)])
-
+    @validate(None, int, int)
     def cofactor(self, row_to_ignore: int, col_to_ignore: int) -> Matrix:
         """will return the cofactor of the matrix at the given position
 
@@ -656,20 +733,21 @@ class Matrix:
         Returns:
             Matrix: the cofactor of the matrix at the given position
         """
-        if not((0 <= row_to_ignore < self.__rows) and (0 <= col_to_ignore < self.__cols)):
+        if not ((0 <= row_to_ignore < self._rows) and (0 <= col_to_ignore < self._cols)):
             raise ValueError("Row or column index out of range")
         res: list[list[Any]] = []
-        for i, row in enumerate(self.__matrix):
+        for i, row in enumerate(self._matrix):
             if i == row_to_ignore:
                 continue
             res.append([])
-            for j, col in enumerate(self.__matrix[i]):
+            for j, col in enumerate(self._matrix[i]):
                 if j == col_to_ignore:
                     continue
                 res[i if i < row_to_ignore else i -
-                    1].append(self.__matrix[i][j])
+                    1].append(self._matrix[i][j])
         return Matrix(res)
 
+    @validate(None, int, int)
     def minor(self, row_to_ignore: int, col_to_ignore: int) -> Any:
         """will return the minor of the matrix at the given position (the determinant of the cofactor)
 
@@ -691,8 +769,8 @@ class Matrix:
         Returns:
             Matrix: the transpose of the matrix
         """
-        return Matrix([[self.__matrix[j][i] for j in range(self.__cols)]
-                       for i in range(self.__rows)])
+        return Matrix([[self._matrix[j][i] for j in range(self._cols)]
+                       for i in range(self._rows)])
 
     def conjugate(self) -> Matrix:
         """will return the conjugate of the matrix
@@ -730,8 +808,8 @@ class Matrix:
                 return 0
             return -1 if a_index > b_index else 1
 
-        self.__matrix = sorted(
-            self.__matrix, key=functools.cmp_to_key(comparer), reverse=True)
+        self._matrix = sorted(
+            self._matrix, key=functools.cmp_to_key(comparer), reverse=True)
 
     def apply_operation(self, operation: MatrixOperationType, iv1, iv2, operate_with=None) -> Matrix:
         if not isinstance(operation, MatrixOperationType):
@@ -755,6 +833,7 @@ class Matrix:
 
         return operate_with
 
+    @validate(None, [[Vector, Matrix__], None, None])
     def concat(self, other: Union[Vector, Matrix]) -> Matrix:
         """will concatenate the matrix with the given matrix and return the result in a new matrix
 
@@ -768,8 +847,6 @@ class Matrix:
         Returns:
             Matrix: the result of the concatenation
         """
-        if not isoneof(other, [Vector, Matrix]):
-            raise TypeError("can only concat with a Vector or Matrix")
         if len(other) != len(self):
             raise ValueError(
                 "can only concat with a Matrix|Vector with the same number of rows")
@@ -780,6 +857,7 @@ class Matrix:
             res.append(self[i] + other[i])
         return Matrix(res)
 
+    @validate(None, int)
     def split(self, index: int) -> Tuple[Matrix, Matrix]:
         """will split the matrix into two matrices at the given index
 
@@ -793,8 +871,6 @@ class Matrix:
         Returns:
             Tuple[Matrix, Matrix]: the two matrices resulting from the split
         """
-        if not isinstance(index, int):
-            raise TypeError("can only split at an int")
         if not 0 <= index <= len(self[0]):
             raise ValueError("index out of range")
         if index == len(self[0]):
@@ -884,6 +960,7 @@ class Matrix:
             arr += [self[row][col] for row in range(len(self))]
         return Vector(arr)
 
+    @validate(None, Vector)
     def solve(self, vec: Vector) -> Union[None, Vector, Span]:
         """will solve the matrix with the given vector and return the solution
 
@@ -977,4 +1054,7 @@ class Matrix:
 
     def geometric_multiplicity(self, eigenvalue) -> int:
         m: Matrix = self-eigenvalue*Matrix.identity(len(self))
-        return m.kernel.dim
+        res = self.kernel
+        if isinstance(res, Vector):
+            res = Span([res])
+        return res.dim
