@@ -3,3 +3,5 @@ from .instance_cheking import *
 from .string_manipulation import *
 from .polynomial_from_string_helper_functions import *
 from .mathematical_helpers import *
+from .math_symbols import *
+from .pretty_print import *
